#include <EEPROM.h>
#include <flash_stm32.h>
/*
   _===_ _           _                 ______ _     _             _ _ _
  |  ___| |         | |                | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ _____| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \_____| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |    | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/     \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Contrôle de l'Atténuateur HP 33322H via STM32-Mini et ULN2803
  Électro-Bidouilleur, Janvier 2017
*/
//------------------------------------------------------------------------------
// Définitions et instructions au compilateur

// Broches vers l'Atténuateur
#define dB10_in_solen     PA0           // Broche d'Activation de l'atténuateur 10dB
#define dB20_in_solen     PA1           // Broche d'Activation de l'atténuateur 20dB
#define dB40_1_in_solen   PA2           // Broche d'Activation de l'atténuateur 40dB #1
#define dB40_2_in_solen   PA3           // Broche d'Activation de l'atténuateur 40dB #2
#define dB10_out_solen    PA4           // Broche de retrait de l'atténuateur 10dB
#define dB20_out_solen    PA5           // Broche de retrait de l'atténuateur 20dB
#define dB40_1_out_solen  PA6           // Broche de retrait de l'atténuateur 40dB #1
#define dB40_2_out_solen  PA7           // Broche de retrait de l'atténuateur 40dB #2
#define atten_incr_pb     PB1           // Broche du bouton poussoir d'augmentation d'Atténuation
#define atten_dec_pb      PB0           // Broche du bouton poussoir de diminution d'Atténuation
// Autres définitions
#define pinLED PC13                     // Broche de la DEL sur la plaquette STM32 Mini
#define LF 10                           // line feed (ASCII-10)

 // Script d'alimentation d'un solénoïde
#define impuls_solen(pin)  {                           \                         
                              digitalWrite(pin, HIGH); \
                              delay(150);              \
                              digitalWrite(pin, LOW);  \
                              delay(10);               \
                           }

 // Script de clignotement de la LED
#define flash_LED(pin) {                               \
                          digitalWrite(pin, LOW);      \
                          delay(50);                   \
                          digitalWrite(pin, HIGH);     \
                       }

 // Script de pulsation d'une broche de sortie
#define pulse_pin(pin) {                               \   
                          digitalWrite(pin, HIGH);     \
                          delayMicroseconds(10);       \
                          digitalWrite(pin, LOW);      \
                          delayMicroseconds(10);       \
                       }

//------------------------------------------------------------------------------
// Tables en mémoire flash de programme

// Table des correspondance des étages d'atténuation vs. la valeur d'atténuation en dB.
const char atten_corresp_table[] PROGMEM = {0b0000,   // 0dB
                                            0b0001,   // 10dB
                                            0b0010,   // 20dB
                                            0b0011,   // 30dB
                                            0b0100,   // 40dB
                                            0b0101,   // 50dB
                                            0b0110,   // 60dB
                                            0b0111,   // 70dB
                                            0b1100,   // 80dB
                                            0b1101,   // 90dB
                                            0b1110,   // 100dB
                                            0b1111};  // 110dB

//------------------------------------------------------------------------------
// Variables Globales

uint16 Atten_Mem_Val      = 0;                  // Valeur d'atténuation courante  (à sauvegarder en mémoire Flash)
uint16 Atten_Mem_Address  = 0x10;               // Addresse de la sauvegarde d'atténuation en mémoire Flash
uint16 Status;                                  // État de l'opération de sauvegarde en mémoire flash.
char Atten_dB             = 0;                  // Variable intermédaire de l'Atténuation

//------------------------------------------------------------------------------
// Procédure d'activation des solénoïdes des atténuateurs

void activ_atten_solen(char nouv_atten_dB)
{
  char atten_solen_masque;                                            // Le masque utilisé pour activer/désactiver les solénoïdes

  atten_solen_masque = atten_corresp_table[nouv_atten_dB / 10];
  if (atten_solen_masque & 0b0001) impuls_solen(dB10_in_solen);       // envoi d'un bit
  if (atten_solen_masque & 0b0010) impuls_solen(dB20_in_solen);       // envoi d'un bit
  if (atten_solen_masque & 0b0100) impuls_solen(dB40_1_in_solen);     // envoi d'un bit
  if (atten_solen_masque & 0b1000) impuls_solen(dB40_2_in_solen);     // envoi d'un bit
  atten_solen_masque = ~atten_solen_masque;
  if (atten_solen_masque & 0b0001) impuls_solen(dB10_out_solen);      // envoi d'un bit
  if (atten_solen_masque & 0b0010) impuls_solen(dB20_out_solen);      // envoi d'un bit
  if (atten_solen_masque & 0b0100) impuls_solen(dB40_1_out_solen);    // envoi d'un bit
  if (atten_solen_masque & 0b1000) impuls_solen(dB40_2_out_solen);    // envoi d'un bit
}

//------------------------------------------------------------------------------
// Procédure de changement d'atténuation

void ajust_atten(int nouv_atten_dB)
{
  Serial.print("   Attenuation demandee (dB): ");                               // Afficher l'atténuation demandée sur le port série
  Serial.println(nouv_atten_dB);
  Atten_Mem_Val = nouv_atten_dB;                                                // Sauvegarder la nouvelle atténuation en mémoire flash
  Status = EEPROM.write(Atten_Mem_Address, Atten_Mem_Val);                      //      "
  if (Status == 0) Serial.println("   Attenuation sauvegardee en memoire.");    // Si sauvegarde réussie, afficher confirmation de sauvegarde sur le port série
  else Serial.println("   Échec de sauvegarde en memoire de l'attenuation.");   // Sinon, afficher échec de sauvegarde sur le port série
  activ_atten_solen(nouv_atten_dB);                                             // Activer les solénoïdes selon l'atténuation demandée.
}

//------------------------------------------------------------------------------
// Fonction de vérification de nombre. Retourne Vrai si la chaîne de caractères représente un nombre

boolean IsNumeric(String str)
{
  if (str.length() == 0)  return false;                     // si la chaîne est vide, pas un nombre
  for (char i = 0; i < str.length(); i++)                   // tester tous les caractères, un à la fois
  {
    if (!(isDigit(str.charAt(i)) ))                         // Est-ce que le caractère est un chiffre?
    {
      return false;                                         // Non, retourner Faux
    }
  }
  return true;                                              // Ici, tous les caractères ont été vérifiés comme étant des chiffres. Retourner Vrai.
}

//------------------------------------------------------------------------------
// Procédure de configuration du STM32, style arduino

void setup()
{
  char i;   // Variable-compteur
  
   // configurations des broches divers
  pinMode(dB10_in_solen, OUTPUT);                 // Sortie
  pinMode(dB20_in_solen, OUTPUT);                 // Sortie
  pinMode(dB40_1_in_solen, OUTPUT);               // Sortie
  pinMode(dB40_2_in_solen, OUTPUT);               // Sortie
  pinMode(dB10_out_solen, OUTPUT);                // Sortie
  pinMode(dB20_out_solen, OUTPUT);                // Sortie
  pinMode(dB40_1_out_solen, OUTPUT);              // Sortie
  pinMode(dB40_2_out_solen, OUTPUT);              // Sortie
  pinMode(atten_incr_pb, INPUT);                  // Entrée
  pinMode(atten_dec_pb, INPUT);                   // Entrée
  pinMode(pinLED, OUTPUT);                        // Sortie

  digitalWrite(pinLED, HIGH);                     // éteindre la DEL

  Serial.begin(9600);                             // configuration du port série (broches A9 et A10)
  Serial.setTimeout(10);                          // Délai d'attente maximum de la fonction serial.read

   // Définir la zone du flash à utiliser comme émulation de EEPROM
  EEPROM.PageBase0 = 0x801F000;
  EEPROM.PageBase1 = 0x801F800;
  EEPROM.PageSize  = 0x400;

   // Mettre à jour l'atténuation selon la valeur sauvegardée.
  Status = EEPROM.read(Atten_Mem_Address, &Atten_Mem_Val);                                  // Lire la valeur sauvegardée en mémoire flash
  if ((Atten_Mem_Val < 0) || (Atten_Mem_Val > 110) || (Status != 0)) Atten_Mem_Val = 110;   // couvre les cas où la mémoire serait vide ou corrompue.
  Serial.print("   Attenuation en Memoire (dB): ");                                         // Afficher l'atténuation sur le port série
  Serial.println(Atten_Mem_Val);                                                            //          "
  activ_atten_solen(Atten_Mem_Val);                                                         // Activer les solénoïdes selon l'atténuation en mémoire. S'assure que la valeur d'atténuation sauvegardée est bien celle appliquée
  Serial.println("Entrez l'attenuation (dB): ");                                            // Afficher la demande d'atténuation à l'usager sur le port série
}

//------------------------------------------------------------------------------
// Boucle principale, style arduino

void loop()
{
   // Définition des variables
  String atten_input = "";                                              // La chaîne de caractères contenant la fréquence
  char temp_char;                                                       // Caractère temporaire

   // Boucle principale
  while (true)                                                          // Boucle perpétuelle
  {
     // Validation d'un changement d'atténuation par port série
    temp_char = Serial.read();                                          // Lire un caractère du port série.  Expiration de la lecture après 10ms
    if (temp_char >= 10 && temp_char <= 127)                            // Validation du caractère lu (est-ce un caractère?)
    {
      if (temp_char != LF) atten_input += temp_char;                    // Ajouter le caractère à la chaîne de commande si la fin de la commande (délimitée par LF (ASCII 10)) n'est pas reçue
      else      // Fin de la commande reçue ici
      {
        Serial.print("   Attenuation entree (dB): ");                   // Afficher sur le port série l'atténuation reçue 
        Serial.println(atten_input);                                    //       "
        if ((IsNumeric(atten_input)) && (atten_input.toInt() >= 0) && (atten_input.toInt() <= 110) && (atten_input.toInt() % 10 == 0))    // Valider si la chaîne de caractères représente un nombre de 0 à 110 en dizaine?
        {
          ajust_atten(atten_input.toInt());                             // Atténuation valide. Ajuster l'atténuation à la nouvelle valeur
          flash_LED(pinLED);                                            // Faire clignoter le LED
        }
        else Serial.println("   Erreur dans le format de l'attenuation entree. Veuillez recommencer...");   // Sinon, mauvaise valeur d'atténuation reçue
        atten_input = "";                                               // Ré-initialiser la chaîne de caractères à vide.
        Serial.println("Entrez l'attenuation (dB): ");                  // Afficher la demande d'atténuation à l'usager sur le port série
      }
    }
     // Validation d'un changement d'atténuation par bouton poussoir
    if (!digitalRead(atten_incr_pb) || !digitalRead(atten_dec_pb))      // Un des bouton enfoncé ?
    {
      flash_LED(pinLED);                                                // Oui, faire clignoter le DEL
      if (!digitalRead(atten_incr_pb) && (Atten_Mem_Val < 110))         // Bouton d'augmentation enfoncé?
      { 
        ajust_atten(Atten_Mem_Val + 10);                                // Oui, augmenter l'atténuation
      }
      else if (!digitalRead(atten_dec_pb) && Atten_Mem_Val > 0)         // Non, Bouton de diminution enfoncé?
      {
        ajust_atten(Atten_Mem_Val - 10);                                // Oui, diminuer l'atténuation
      }
      Serial.println("Entrez l'attenuation (dB): ");                    // Afficher la demande d'atténuation à l'usager sur le port série
    }
  }
}
